import ctypes 
import tisgrabber as tis
import cv2
import numpy as np

#tisgrabber_x64.dllをインポートする
ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)

#ICImagingControlクラスライブラリを初期化します。
#この関数は、このライブラリの他の関数が呼び出される前に1回だけ呼び出す必要があります。
ic.IC_InitLibrary(0)

#ダイアログ画面を表示
hGrabber = tis.openDevice(ic)

#デバイスが有効か確認
if(ic.IC_IsDevValid(hGrabber)):
    #ライブスタート開始　引数：0の時非表示、引数：1の時表示
    #ライブ画面は非表示にしています。
    ic.IC_StartLive(hGrabber, 0)
    key = ""
    while key != "q":
        print("pキー: 画像処理を実行")
        print("qキー: プログラムを終了")
        key = input('pキーかqキーを押下してください。:')
        if key == "p":
            if ic.IC_SnapImage(hGrabber, 2000) == tis.IC_SUCCESS:
                # 画像記述の変数を宣言する
                Width = ctypes.c_long()
                Height = ctypes.c_long()
                BitsPerPixel = ctypes.c_int()
                colorformat = ctypes.c_int()

                # 画像の説明の値をクエリする
                ic.IC_GetImageDescription(hGrabber, Width, Height,
                                          BitsPerPixel, colorformat)

                # バッファサイズを計算
                bpp = int(BitsPerPixel.value / 8.0)
                buffer_size = Width.value * Height.value * bpp

                # イメージデータを取得する
                imagePtr = ic.IC_GetImagePtr(hGrabber)

                imagedata = ctypes.cast(imagePtr,
                                        ctypes.POINTER(ctypes.c_ubyte *
                                                       buffer_size))

                # numpyの配列を作成する
                image = np.ndarray(buffer=imagedata.contents,
                                   dtype=np.uint8,
                                   shape=(Height.value,
                                          Width.value,
                                          bpp))

                # 画像にOpenCV関数を適用
                image = cv2.flip(image, 0)
                image = cv2.erode(image, np.ones((11, 11)))

                #表示を縮小
                resized_img = cv2.resize(image,(640, 480)) 
                cv2.imshow('Window', resized_img)
                cv2.waitKey(10)
            else:
                print("2秒間フレームが送られてきませんでした。")

    ic.IC_StopLive(hGrabber)

    cv2.destroyWindow('Window')

else:
    ic.IC_MsgBox(tis.T("No device opened"), tis.T("Simple Live Video"))

ic.IC_ReleaseGrabber(hGrabber)
